# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import winrt.system
from winrt._winrt_windows_devices_bluetooth_rfcomm import (
    RfcommDeviceService,
    RfcommDeviceServicesResult,
    RfcommServiceId,
    RfcommServiceProvider,
)

__all__ = [
    "RfcommDeviceService",
    "RfcommDeviceServicesResult",
    "RfcommServiceId",
    "RfcommServiceProvider",
]

